window.addEventListener("load", mccInit, false);

// if false, no log messages will be done
const DEBUG = true;
const UPDATE_REQUENCE = 1800000; // 30 minutes

// multexi settings
var gEdfUrl = "";

/************************************************************
 *						INSTALL/UPDATES						*
 ************************************************************/
////////////////////////////////////////////////////////////
// Observes the updates progress.
function MCCUpdateItemObserver()
{
  this.register();
}

MCCUpdateItemObserver.prototype = {
	_items: [],
	
  observe: function(subject, topic, data) {
  	var finished = false;
  	
	switch (topic) {
	case "Update:Extension:Started":
		mccStatusOutput("Checking for updates");
		
		if(mccIsDialogEnabled("installUpdateWarning")) {
			window.openDialog("chrome://multexi/content/install-warning-dialog.xul",
	                  "installUpdateWarning",
	                  "chrome,dialog,modal");
        }
	break;
	
	case "Update:Extension:Item-Started":
    	if (subject) {
        	var item = subject.QueryInterface(Components.interfaces.nsIUpdateItem);
        	mccStatusOutput("Checking: " + item.name);
      	}
      break;

	case "Update:Extension:Item-Ended":
    	if ((subject) && (data == "update-check-success")) {
        	var item = subject.QueryInterface(Components.interfaces.nsIUpdateItem);
        	this._items.push(item.xpiURL);
        	mccStatusOutput("Out of date: " + item.name);
      	}

      break;
    case "Update:Extension:Ended":
    	mccStatusOutput("Update check finished");
    	finished = true;
      break;
    }

    if(finished) {
    	this.unregister();
    	
    	if(this._items.length > 0)
	    	gInstallUpdate.registerItems(this._items);

    	// install all loaded items
    	gInstallUpdate.doInstall();
    }
  },
  
  register: function() {
  	var observer = null;
  	// Observer           
	observer = Components.classes["@mozilla.org/observer-service;1"]
            .getService(Components.interfaces.nsIObserverService);
    
    observer.addObserver(this, "Update:Extension:Started", false);        
    observer.addObserver(this, "Update:Extension:Item-Started", false);
	observer.addObserver(this, "Update:Extension:Ended", false);
	observer.addObserver(this, "Update:Extension:Item-Ended", false);
  },
  
  unregister: function() {
  	var observer = null;
  	// Observer           
	observer = Components.classes["@mozilla.org/observer-service;1"]
            .getService(Components.interfaces.nsIObserverService);
            
	observer.removeObserver(this, "Update:Extension:Started");        
    observer.removeObserver(this, "Update:Extension:Item-Started");
	observer.removeObserver(this, "Update:Extension:Ended");
	observer.removeObserver(this, "Update:Extension:Item-Ended");
  },
}

////////////////////////////////////////////////////////////
// responsible for the initialization of a new install/
// update progress, using the nsIXPInstallManager and the
// process controlling of a new installation.
var gInstallUpdate = {
	_installItems: [],
	_downloaded: 0,
	_installed: 0,
	
	registerItems: function (items) {
		this._installItems = 
			this._installItems.concat(items);
			
		return;
	},
	
	doInstall: function () {
		// convert update items to url
		var xpInstallManager = null;

		if(this._installItems.length == 0) {
			return;
		}

		xpInstallManager = Components.classes["@mozilla.org/xpinstall/install-manager;1"]
	                           .getService(Components.interfaces.nsIXPInstallManager);

		// invocate installation
		xpInstallManager.
			initManagerFromChrome(this._installItems, this._installItems.length, this);
	},
	
	removeMismatch: function () {
		var extManager = Components.classes["@mozilla.org/extensions/manager;1"]
	                       .getService(Components.interfaces.nsIExtensionManager);
	                       
	   	return extManager.checkForMismatches();
	},
	
	operationFinished: function(installed) {
		mccStatusOutput("MultExI finished");
		
		/* remove incompatible extensions */
		var mismRestart = this.removeMismatch();
		
		if((installed) || (mismRestart)) {
			/* shutdown application */
			var appStartup = Components.classes["@mozilla.org/toolkit/app-startup;1"]
                       .getService(Components.interfaces.nsIAppStartup);
 
            window.openDialog("chrome://multexi/content/restart-message-dialog.xul",
              "restartMessage",
              "chrome,dialog,modal");

            appStartup.quit(Components.interfaces.nsIAppStartup.eAttemptQuit);
		}

		this._installItems = [];
		this._downloaded = 0;
		this._installed = 0;
	},

	onStateChange: function (aIndex, aState, aValue)
	{
		const nsIXPIProgressDialog = Components.interfaces.nsIXPIProgressDialog;
		
		switch (aState) {
		case nsIXPIProgressDialog.DOWNLOAD_START:
			this._downloaded++;
			mccStatusOutput("Downloading: " + 
								this._downloaded + "/" + this._installItems.length);
			break;
		case nsIXPIProgressDialog.DOWNLOAD_DONE:
			break;
		case nsIXPIProgressDialog.INSTALL_START:
			mccStatusOutput("Installing: " + 
							(this._installed+1) + "/" + this._installItems.length);
			break;
		case nsIXPIProgressDialog.INSTALL_DONE:
			this._installed++;
			break;
		case nsIXPIProgressDialog.DIALOG_CLOSE:
			this.operationFinished(this._installed > 0);
			break;
		}
	},

	onProgress: function (aIndex, aValue, aMaxValue)
	{
		return;
	}
}

/************************************************************
 *					USER INTERACTION						*
 ************************************************************/

////////////////////////////////////////////////////////////
// checks if a specific window is enabled
function mccIsDialogEnabled(aName) {
	var prefs = null;
	var showDlg = true;
	
	// get preference branch
	prefs = Components.classes["@mozilla.org/preferences-service;1"].
                   getService(Components.interfaces.nsIPrefService);
	prefs = prefs.getBranch("extensions.multexi.");
	
	// check if show dlg
	try {
		showDlg = prefs.getBoolPref("dlg." + aName + ".show");
	} catch (e) {
	}
	
	return showDlg;
}

////////////////////////////////////////////////////////////
// handles the default user information dialogs
var gDlgDefault = {

	onPageShow: function() {
		// prepare dialog
		document.documentElement.getButton("back").
				setAttribute("hidden", "true");
				
		document.documentElement.getButton("cancel").
				setAttribute("hidden", "true");

		document.documentElement.getButton("finish").focus();
		
		return true;
	},
	
	onPageHide: function () {
		var prefs = null;
		var showAgainChbx = null;
		var id = "undefined";
		
		try {
			showAgainChbx = document.getElementById("dontShowAgain");
		} catch (e) {
		}
		
		id = document.documentElement.getAttribute('id');

		if(showAgainChbx != null) {
			// get preference branch
			prefs = Components.classes["@mozilla.org/preferences-service;1"].
	                    getService(Components.interfaces.nsIPrefService);
			prefs = prefs.getBranch("extensions.multexi.");
		
			// set show dialog again	
			prefs.setBoolPref("dlg." + id + ".show", 
					!showAgainChbx.getAttribute("checked"));
		}
		
		return true;
	},
	
};


////////////////////////////////////////////////////////////
// puts out a log message into javascript console
function mccLog(message) {
	if(DEBUG) {
		var consoleLogger = Components.classes['@mozilla.org/consoleservice;1'].
								getService(Components.interfaces.nsIConsoleService);
	    consoleLogger.logStringMessage('MultExI: ' + message);
	}
}

////////////////////////////////////////////////////////////
// updates the status string in the browser status bar
function mccStatusOutput (aString) {
	element = document.getElementById("multexi-status-label");
	element.value = aString;
}

/************************************************************
 *						UTIL FUNCTIONS						*
 ************************************************************/
////////////////////////////////////////////////////////////
// verifies a XmlRequest.
function mccIsXmlValid (aXmlReq) {
	return aXmlReq.responseXML.documentElement.nodeName != "parsererror";
}

/************************************************************
 *						CONFIGURATION						*
 ************************************************************/
 
/*
 * DESTROY CONFIG
 */
////////////////////////////////////////////////////////////
// stores configuration preferences from package
// included configuration file.
function mccDestroyPkgCfg () {
	var prefs = null;

	// create preference branch
	prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);
	prefs = prefs.getBranch("extensions.multexi.");
	
	prefs.setBoolPref("client.pkgCfgStored", false);
	prefs.setBoolPref("client.cfgStored", false);
}

/*
 * WRITE CONFIG
 */
 
////////////////////////////////////////////////////////////
// stores configuration preferences from package
// included configuration file.
function mccStorePackageCfg (aPrefBranch) {
	mccLog("Storing package configuratioN");
	
	// write cfg stored
	aPrefBranch.setBoolPref("client.pkgCfgStored", true);
	// write edf url
	aPrefBranch.setCharPref("server.edfUrl", gEdfUrl);
}

////////////////////////////////////////////////////////////
// stores configuration preferences from external
// configuration file and extension internal configurations.
function mccStoreCfg (aPrefBranch) {
	mccLog("Storing configuration");
	
	// store variables
	aPrefBranch.setBoolPref("client.cfgStored", true);
}

/*
 * READ CONFIG
 */

////////////////////////////////////////////////////////////
// Reads configuration that has been delivered with multexi.
// It does _not_ store this configuration in the applications
// property branch.
function mccReadPackageCfg (aPrefBranch) {
	var cfgStored = false;
	var xmlReq = null;
	var dom = null;
	var xpathResult = null;
	
	try {
		cfgStored = aPrefBranch.getBoolPref("client.pkgCfgStored");
	} catch (e) {}
	
	if(!cfgStored) {
		// load configuration from local file
		xmlReq = new XMLHttpRequest();
		
		xmlReq.open("GET", "chrome://multexi/content/mcc_config.xml", false);
		xmlReq.send(null);
		
		if(!mccIsXmlValid(xmlReq)) {
			mccLog("Couldn't read chrome://multexi/content/mcc_config.xml");
			return false;
		}
		
		dom = xmlReq.responseXML;
		
		/*
		 * read edf url
         */
        xpathResult = dom.evaluate("/mcc/edf-url", 
								dom, null, XPathResult.ANY_TYPE, null);
		xpathResult = xpathResult.iterateNext();
		gEdfUrl = xpathResult.textContent;
	} else {
		// load configuration from pref branch
		try {
			gEdfUrl = aPrefBranch.getCharPref("server.edfUrl");
		} catch (e) {}
	}

	// check edf url
	if((gEdfUrl == null) || (gEdfUrl.length <= 0)) {
		mccLog("Couldn't read edf-url");
		return false;
	}
	
	// print variables
	mccLog("EDF URL:" + gEdfUrl);

	return true;
}

////////////////////////////////////////////////////////////
// Reads external configuration if local copy is not available
// or if external config. has been updated.
function mccReadCfg (aPrefBranch) {
	mccLog("Skipping mccReadCfg");
	
	return true;
}

////////////////////////////////////////////////////////////
// Initializes the reading of all configurations.
function mccInitCfg () {
	var prefs = null;

	// create preference branch
	prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);
	prefs = prefs.getBranch("extensions.multexi.");
	
	// check package configuration
	if(!mccReadPackageCfg(prefs)) {
		mccLog("Error while reading package cfg");
		return false;
	}
	
	// check remote configruation
	if(!mccReadCfg()) {
		mccLog("Error while reading cfg");
		return false;
	}
	
	// store all cfg parameters now
	mccStorePackageCfg(prefs);
	mccStoreCfg(prefs);
	
	return true;
}

////////////////////////////////////////////////////////////
// initializes the extension (concerning configuration)
function mccInit () {
	var prefs = null;
	var lastRun = 0;
	var now = new Date().getTime();
	var timeDiff = 0;
	
	mccLog("MultExI startup");
	
	// check if multexi should run
	prefs = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefService);
	prefs = prefs.getBranch("extensions.multexi.");
	
	try {
		lastRun = prefs.getCharPref("lastRun");
	} catch (e) {
	}
	
	timeDiff = now - lastRun;

	if(timeDiff < UPDATE_REQUENCE) {
		// now update
		mccLog("MultExI won't update now");
		mccUpdateRunTimeout();
		return true;
	} else {
		prefs.setCharPref("lastRun", "" + now);
	}
	
	// read configuration
	if(!mccInitCfg()) {
		mccLog("Could not read configuration. Exiting");
		return;
	}
	
	mccSynchronizeMsc();
}

/************************************************************
 *						MULTEXI								*
 ************************************************************/
 
////////////////////////////////////////////////////////////
// refreshes the timeout which is responsible for the
// cyclical run of MultExI
function mccUpdateRunTimeout () {
	mccLog("Initiating cyclic run");
	window.setTimeout('mccInit()', UPDATE_REQUENCE + 100);
}

////////////////////////////////////////////////////////////
// synchronizes installed extensions with extensions
// defined in servers edf.xml file.
function mccSynchronizeMsc () {
	var xmlReq = new XMLHttpRequest();
	var dom = null;
	var xpathResult = null;
	var xpathResultInner = null;
	var xmlExtDef = null;
	var updateObserver = null;
	var extItemsUrls = null;
	var extensionManager = null;
	var updateItemList = null;
	var nsIUpdateItem  = null;
	
	// extension manager
	extensionManager = Components.classes["@mozilla.org/extensions/manager;1"].
						getService(Components.interfaces.nsIExtensionManager);
	
	// update item
	
	nsIUpdateItem = Components.interfaces.nsIUpdateItem;

	mccLog("Synchronizing with MSC at " + gEdfUrl);
	
	// request edf.xml from server
	xmlReq.open("GET", gEdfUrl, false);
	xmlReq.send(null);

	// verify configuration
	if(!mccIsXmlValid(xmlReq)) {
		// error occured
		mccLog("Error occured when requesting: " + gEdfUrl);
		
		// perhaps we can help our self if we
		// re-read the pkg cfg(?).
		mccDestroyPkgCfg();
		
		return false;
	}
	
	// get item list
	updateItemList = extensionManager.
				getItemList(nsIUpdateItem.TYPE_EXTENSION, {});
	
	// assign dom
	dom = xmlReq.responseXML;
	
	var count = dom.evaluate("count(/edf/extensions/extension)", 
							dom, null, XPathResult.NUMBER_TYPE, null);

	// read extensions
	xpathResult = dom.evaluate("/edf/extensions/extension", 
								dom, null, XPathResult.ANY_TYPE, null);

	var instItems = [];
	var avItems = [];
	while(xmlExtDef = xpathResult.iterateNext()) {
		var extItem = null;
		var extUrl = null;
		var extGuid = null;
		var extFound = false;
		
		// get url
		xpathResultInner = dom.evaluate("child::url", 
								xmlExtDef, null, XPathResult.ANY_TYPE, null);
		extUrl = xpathResultInner.iterateNext().textContent;
		
		// get guid
		xpathResultInner = dom.evaluate("child::guid", 
								xmlExtDef, null, XPathResult.ANY_TYPE, null);
		extGuid = xpathResultInner.iterateNext().textContent;
		
		// log msg
		mccLog("Checking " + extUrl + " : " + extGuid);
		
		// extension exists?
		for(i = 0; i < updateItemList.length; i++) {
			if(updateItemList[i].id == extGuid) {
				extFound = true;
			}
		}

		if(!extFound) {
			// extension not found .. install it
			mccLog("Extension " + extUrl + " : " + extGuid + " not found");
			
			// add this item to the array of extensions to install
			instItems.push(extUrl);
		} else {
			mccLog("Extension " + extGuid + " found");
			avItems.push(extensionManager.getItemForID(extGuid));
		}
	}

	if(instItems.length > 0) {
		/*
		 *	Install not installed items
		 */
		gInstallUpdate.registerItems(instItems);
	}

	if(avItems.length > 0) {
		// check for updates
		var observer = new MCCUpdateItemObserver();
		extensionManager.update(avItems, avItems.length, false, observer);
	}
	
	// ensure next run
	mccUpdateRunTimeout();

	return true;
}
